@@ -1,11 +1,13 @@
 ## File Name: amh_sampling.R
-## File Version: 0.23
+## File Version: 0.393
 
-##################################################
-# sampling step in amh routine
+
+#--- sampling step in amh routine
 amh_sampling <- function(pars, data, model, prior,
                 proposal_sd, acceptance_parameters,
-                pars_lower, pars_upper, pmle_pars, eps=1E-100 ){
+                pars_lower, pars_upper, pmle_pars, eps=1E-100, it=NULL,
+                boundary_ignore=FALSE)
+{
     NP <- length(pars)
     priorval0 <- 0
     for (pp in 1:NP){
@@ -18,12 +20,14 @@ amh_sampling <- function(pars, data, model, prior,
         priorval_pp_old <- log( do.call( prior[[pp]][[1]], prior_arg_pp ) + eps )
         #*** evaluate posterior for new parameters
         pars_pp_new <- stats::rnorm( 1, mean=pars[pp], sd=proposal_sd[pp] )
-        if ( pars_pp_new < pars_lower[pp] ){
-            pars_pp_new <- pars_lower[pp]
-        }
-        if ( pars_pp_new > pars_upper[pp] ){
-            pars_pp_new <- pars_upper[pp]
-        }
+
+        #* handle sampled values at boundaries
+        res <- amh_sampling_boundary_values(pars_pp_new=pars_pp_new, pp=pp,
+                pars_lower=pars_lower,    pars_upper=pars_upper, pars_old=pars_old,
+                boundary_ignore=boundary_ignore)
+        reject <- res$reject
+        pars_pp_new <- res$pars_pp_new
+
         pars_new <- pars
         pars_new[pp] <- pars_pp_new
         ll_new <- do.call( model, list( pars=pars_new, data=data ) )
@@ -34,6 +38,9 @@ amh_sampling <- function(pars, data, model, prior,
         mh_prob <- exp( ( ll_new + priorval_pp_new ) -  ( ll_old + priorval_pp_old ) )
         # acceptance or rejection
         rand <- stats::runif(1)
+        if (reject){
+            mh_prob <- 0
+        }
         if ( rand < mh_prob ){
             accept <- 1
             pars <- pars_new
@@ -47,7 +54,6 @@ amh_sampling <- function(pars, data, model, prior,
             ll <- ll_old
             priorval0 <- priorval0 + priorval_pp_old
         }
-
         acceptance_parameters[ pp, 1:2] <- acceptance_parameters[ pp, 1:2] + c( accept, 1 )
     }  # end pp
     #---- posterior update
@@ -64,4 +70,3 @@ amh_sampling <- function(pars, data, model, prior,
                         posteriorval=posteriorval, pmle_pars=pmle_pars)
     return(res)
 }
-##############################################################################
